#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   robot.py
@Time    :   2025/12/16 16:39:44
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   双臂机器人控制接口
'''

from typing import Optional, List, Dict, Tuple
from .arm import Arm
from .exceptions import (
    CANInitializationError,
    InvalidMotorIDError,
    InvalidModeError
)
from ._lib.can_utils import (
    verify_can_interface,
    enable_can_interface
)
from ._lib.log_utils import log_output


class Robot:
    """
    双臂机器人控制类

    管理左右两条机械臂，提供统一的控制接口。

    属性:
        left_arm (Arm): 左臂控制对象
        right_arm (Arm): 右臂控制对象

    示例:
        >>> robot = Robot(left_canid='can0', right_canid='can1')
        >>> robot.enable_all()
        >>> robot.set_mode_all('mit')
        >>> robot.move_all_to_zero_mit(kp=5.0, kd=0.5)
    """

    def __init__(self,
                 left_canid: str = 'can0',
                 right_canid: str = 'can1',
                 motor_ids: Optional[List[int]] = None,
                 auto_enable_can: bool = True,
                 bitrate: int = 1000000,
                 log=None,
                 **kwargs):
        """
        初始化双臂机器人

        参数:
            left_canid (str): 左臂 CAN 通道 (默认: 'can0')
            right_canid (str): 右臂 CAN 通道 (默认: 'can1')
            motor_ids (List[int], optional): 电机ID列表 (默认: [1,2,3,4,5,6,7,8])
            auto_enable_can (bool): 是否自动启用 CAN 接口 (默认: True)
            bitrate (int): CAN 波特率 (默认: 1000000)
            log (callable, optional): 日志函数
            **kwargs: 传递给 Arm 的其他参数

        异常:
            CANInitializationError: CAN 总线初始化失败
        """
        self.left_canid = left_canid
        self.right_canid = right_canid
        self.log = log

        # 检测并启用 CAN 接口
        if auto_enable_can:
            self._check_and_enable_can(left_canid, bitrate)
            self._check_and_enable_can(right_canid, bitrate)

        # 初始化左右臂
        self.left_arm = Arm(
            can_channel=left_canid,
            side='left',
            motor_ids=motor_ids,
            **kwargs
        )
        self.right_arm = Arm(
            can_channel=right_canid,
            side='right',
            motor_ids=motor_ids,
            **kwargs
        )

        # 机械臂列表（便于批量操作）
        self.arms = [self.left_arm, self.right_arm]
        self.arm_names = ['左臂', '右臂']

    def _check_and_enable_can(self, interface: str, bitrate: int = 1000000):
        """
        检测并启用 CAN 接口

        参数:
            interface (str): CAN 接口名称
            bitrate (int): 波特率

        异常:
            CANInitializationError: 启用失败
        """
        # 检查接口是否已经启用
        if verify_can_interface(interface):
            log_output(f"✓ {interface} 已启用", "SUCCESS", self.log)
            return

        # 尝试启用接口
        log_output(f"⚠ {interface} 未启用，正在尝试启用...", "WARNING", self.log)
        success = enable_can_interface(interface, bitrate=bitrate, verbose=False)

        if not success:
            raise CANInitializationError(
                f"无法启用 {interface}。请手动执行:\n"
                f"  sudo ip link set {interface} up type can bitrate {bitrate}"
            )

        log_output(f"✓ {interface} 启用成功", "SUCCESS", self.log)

    def __enter__(self):
        """支持 with 语句"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """退出时自动关闭"""
        self.shutdown()

    def shutdown(self):
        """关闭所有 CAN 总线连接"""
        for arm in self.arms:
            try:
                arm.bus.shutdown()
            except:
                pass

    # ==================== 使能/失能控制 ====================

    def enable_all(self, motor_ids: Optional[List[int]] = None):
        """
        使能所有机械臂的所有电机

        参数:
            motor_ids (List[int], optional): 指定电机ID列表，默认为所有电机
        """
        for arm in self.arms:
            if motor_ids is None:
                ids = arm.motor_ids
            else:
                ids = motor_ids
            for motor_id in ids:
                arm.enable(motor_id)

    def disable_all(self, motor_ids: Optional[List[int]] = None):
        """
        失能所有机械臂的所有电机

        参数:
            motor_ids (List[int], optional): 指定电机ID列表，默认为所有电机
        """
        for arm in self.arms:
            if motor_ids is None:
                ids = arm.motor_ids
            else:
                ids = motor_ids
            for motor_id in ids:
                arm.disable(motor_id)

    def enable_left(self, motor_ids: Optional[List[int]] = None):
        """使能左臂所有电机"""
        ids = motor_ids if motor_ids else self.left_arm.motor_ids
        for motor_id in ids:
            self.left_arm.enable(motor_id)

    def enable_right(self, motor_ids: Optional[List[int]] = None):
        """使能右臂所有电机"""
        ids = motor_ids if motor_ids else self.right_arm.motor_ids
        for motor_id in ids:
            self.right_arm.enable(motor_id)

    def disable_left(self, motor_ids: Optional[List[int]] = None):
        """失能左臂所有电机"""
        ids = motor_ids if motor_ids else self.left_arm.motor_ids
        for motor_id in ids:
            self.left_arm.disable(motor_id)

    def disable_right(self, motor_ids: Optional[List[int]] = None):
        """失能右臂所有电机"""
        ids = motor_ids if motor_ids else self.right_arm.motor_ids
        for motor_id in ids:
            self.right_arm.disable(motor_id)

    # ==================== 模式设置 ====================

    def set_mode_all(self, mode: str, motor_ids: Optional[List[int]] = None):
        """
        设置所有机械臂的控制模式

        参数:
            mode (str): 控制模式 ('mit', 'csp', 'pp', 'speed', 'current')
            motor_ids (List[int], optional): 指定电机ID列表
        """
        for arm in self.arms:
            ids = motor_ids if motor_ids else arm.motor_ids
            for motor_id in ids:
                arm.set_mode(motor_id, mode)

    def set_mode_left(self, mode: str, motor_ids: Optional[List[int]] = None):
        """设置左臂控制模式"""
        ids = motor_ids if motor_ids else self.left_arm.motor_ids
        for motor_id in ids:
            self.left_arm.set_mode(motor_id, mode)

    def set_mode_right(self, mode: str, motor_ids: Optional[List[int]] = None):
        """设置右臂控制模式"""
        ids = motor_ids if motor_ids else self.right_arm.motor_ids
        for motor_id in ids:
            self.right_arm.set_mode(motor_id, mode)

    # ==================== MIT 模式控制 ====================

    def move_all_to_zero_mit(self, kp: float = 5.0, kd: float = 0.5,
                            motor_ids: Optional[List[int]] = None):
        """
        所有机械臂回到零位 (MIT模式)

        参数:
            kp (float): 位置增益
            kd (float): 速度增益
            motor_ids (List[int], optional): 指定电机ID列表
        """
        for arm in self.arms:
            ids = motor_ids if motor_ids else arm.motor_ids
            for motor_id in ids:
                arm.move_to_zero_mit(motor_id, kp=kp, kd=kd)

    def move_joints_mit(self,
                       left_positions: Optional[List[float]] = None,
                       right_positions: Optional[List[float]] = None,
                       kp: float = 5.0,
                       kd: float = 0.5):
        """
        同时控制左右臂关节位置 (MIT模式)

        参数:
            left_positions (List[float], optional): 左臂各关节位置 [pos1, pos2, ...]
            right_positions (List[float], optional): 右臂各关节位置
            kp (float): 位置增益
            kd (float): 速度增益

        示例:
            >>> # 左右臂对称运动
            >>> robot.move_joints_mit(
            >>>     left_positions=[0.1, 0.2, 0.3, 0.4, 0, 0, 0],
            >>>     right_positions=[0.1, 0.2, 0.3, 0.4, 0, 0, 0],
            >>>     kp=10.0, kd=1.0
            >>> )
        """
        if left_positions:
            for i, pos in enumerate(left_positions):
                motor_id = i + 1
                if motor_id in self.left_arm.motor_ids:
                    self.left_arm.move_joint_mit(motor_id, position=pos, kp=kp, kd=kd)

        if right_positions:
            for i, pos in enumerate(right_positions):
                motor_id = i + 1
                if motor_id in self.right_arm.motor_ids:
                    self.right_arm.move_joint_mit(motor_id, position=pos, kp=kp, kd=kd)

    # ==================== CSP 模式控制 ====================

    def move_joints_csp(self,
                       left_positions: Optional[List[float]] = None,
                       right_positions: Optional[List[float]] = None):
        """
        同时控制左右臂关节位置 (CSP模式)

        参数:
            left_positions (List[float], optional): 左臂各关节位置
            right_positions (List[float], optional): 右臂各关节位置
        """
        if left_positions:
            for i, pos in enumerate(left_positions):
                motor_id = i + 1
                if motor_id in self.left_arm.motor_ids:
                    self.left_arm.move_joint_csp(motor_id, position=pos)

        if right_positions:
            for i, pos in enumerate(right_positions):
                motor_id = i + 1
                if motor_id in self.right_arm.motor_ids:
                    self.right_arm.move_joint_csp(motor_id, position=pos)

    # ==================== 状态查询 ====================

    def get_all_status(self) -> Dict[str, Dict]:
        """
        获取所有机械臂的状态

        返回:
            dict: {
                'left': {motor_id: status_info, ...},
                'right': {motor_id: status_info, ...}
            }
        """
        return {
            'left': self.left_arm.get_all_status(),
            'right': self.right_arm.get_all_status()
        }

    def get_left_status(self, motor_id: Optional[int] = None) -> Dict:
        """获取左臂状态"""
        if motor_id:
            return self.left_arm.get_status(motor_id)
        return self.left_arm.get_all_status()

    def get_right_status(self, motor_id: Optional[int] = None) -> Dict:
        """获取右臂状态"""
        if motor_id:
            return self.right_arm.get_status(motor_id)
        return self.right_arm.get_all_status()

    def show_all_status(self):
        """显示所有机械臂的状态"""
        log_output("="*120, "INFO", self.log)
        log_output("左臂状态 (can0)", "INFO", self.log)
        log_output("="*120, "INFO", self.log)
        self.left_arm.show_motor_status(show_header=True)

        log_output("\n" + "="*120, "INFO", self.log)
        log_output("右臂状态 (can1)", "INFO", self.log)
        log_output("="*120, "INFO", self.log)
        self.right_arm.show_motor_status(show_header=True)
        log_output("="*120, "INFO", self.log)

    def show_left_status(self):
        """显示左臂状态"""
        self.left_arm.show_motor_status()

    def show_right_status(self):
        """显示右臂状态"""
        self.right_arm.show_motor_status()

    # ==================== 零点设置 ====================

    def set_zero_all(self, motor_ids: Optional[List[int]] = None):
        """设置所有机械臂的零点"""
        for arm in self.arms:
            ids = motor_ids if motor_ids else arm.motor_ids
            for motor_id in ids:
                arm.set_zero(motor_id)

    def set_zero_left(self, motor_ids: Optional[List[int]] = None):
        """设置左臂零点"""
        ids = motor_ids if motor_ids else self.left_arm.motor_ids
        for motor_id in ids:
            self.left_arm.set_zero(motor_id)

    def set_zero_right(self, motor_ids: Optional[List[int]] = None):
        """设置右臂零点"""
        ids = motor_ids if motor_ids else self.right_arm.motor_ids
        for motor_id in ids:
            self.right_arm.set_zero(motor_id)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   __init__.py
@Time    :   2025/12/16
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   OpenArmX Driver - 内部库模块
'''

# _lib 是内部实现模块，不对外暴露
# 所有公开接口通过 Arm 和 Robot 类提供



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   test_robot.py
@Time    :   2025/12/16
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   Robot 类测试示例（自动检测并启用 CAN）
'''

import sys
import os
import time

# 添加 src 目录到 Python 路径
current_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(current_dir)
src_dir = os.path.join(parent_dir, 'src')
sys.path.insert(0, src_dir)

from openarmx_driver import Robot


if __name__ == "__main__":
    print("="*60)
    print("Robot 类测试 - 自动 CAN 检测和启用")
    print("="*60)

    # Robot 会自动检测并启用 CAN 接口
    # 如果 can0 或 can1 未启用，会自动尝试启用
    with Robot(left_canid='can0', right_canid='can1') as robot:

        # 使能所有电机
        robot.enable_all()

        # 设置为 MIT 模式
        robot.set_mode_all('mit')

        # 显示所有电机状态
        robot.show_all_status()

        # 左右臂对称运动
        robot.move_joints_mit(
            left_positions=[0.1, 0.2, 0.3, 0, 0, 0, 0],
            right_positions=[0.1, 0.2, 0.3, 0, 0, 0, 0],
            kp=10.0,
            kd=1.0
        )
        time.sleep(2)

        # 回到零位
        robot.move_all_to_zero_mit(kp=5.0, kd=0.5)
        time.sleep(1)

        # 停止所有电机
        robot.disable_all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   __init__.py
@Time    :   2025/12/16
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   OpenArmX Driver - Python SDK for OpenArmX robotic arm control
'''

from .arm import Arm
from .robot import Robot
from .exceptions import (
    # 基础异常
    OpenArmXError,

    # CAN 通信异常
    CANError,
    CANInitializationError,
    CANTimeoutError,
    CANTransmissionError,

    # 电机异常
    MotorError,
    MotorNotEnabledError,
    MotorFaultError,
    MotorTimeoutError,
    MotorCalibrationError,

    # 配置异常
    ConfigurationError,
    InvalidMotorIDError,
    InvalidModeError,
    InvalidParameterError,
    ConfigFileError,

    # 限制异常
    LimitExceededError,
    PositionLimitError,
    VelocityLimitError,
    TorqueLimitError,
    KpLimitError,
    KdLimitError,

    # 连接异常
    ConnectionError,
    ConnectionLostError,
)

__version__ = '1.0.1'
__author__ = 'Wei Lindong'

__all__ = [
    # 主接口
    'Arm',
    'Robot',

    # 异常类
    'OpenArmXError',
    'CANError',
    'CANInitializationError',
    'CANTimeoutError',
    'CANTransmissionError',
    'MotorError',
    'MotorNotEnabledError',
    'MotorFaultError',
    'MotorTimeoutError',
    'MotorCalibrationError',
    'ConfigurationError',
    'InvalidMotorIDError',
    'InvalidModeError',
    'InvalidParameterError',
    'ConfigFileError',
    'LimitExceededError',
    'PositionLimitError',
    'VelocityLimitError',
    'TorqueLimitError',
    'KpLimitError',
    'KdLimitError',
    'ConnectionError',
    'ConnectionLostError',
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   exceptions.py
@Time    :   2025/12/16
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   OpenArmX Driver 异常类定义
             定义了所有可能抛出的自定义异常
'''


# ==================== 基础异常类 ====================

class OpenArmXError(Exception):
    """
    OpenArmX 驱动的基础异常类

    所有自定义异常都继承自此类，便于统一捕获和处理。

    示例:
        >>> try:
        >>>     arm.enable(motor_id=5)
        >>> except OpenArmXError as e:
        >>>     print(f"OpenArmX 错误: {e}")
    """
    pass


# ==================== CAN 通信相关异常 ====================

class CANError(OpenArmXError):
    """
    CAN 总线错误基类

    所有与 CAN 总线通信相关的异常都继承自此类。
    """
    pass


class CANInitializationError(CANError):
    """
    CAN 总线初始化失败异常

    当 CAN 总线无法初始化时抛出此异常。
    可能的原因：
    - CAN 接口不存在（如 can0 未配置）
    - 权限不足
    - 驱动未加载
    - 硬件故障

    示例:
        >>> try:
        >>>     arm = Arm('can9')  # can9 不存在
        >>> except CANInitializationError as e:
        >>>     print(f"CAN 初始化失败: {e}")
    """
    pass


class CANTimeoutError(CANError):
    """
    CAN 通信超时异常

    当 CAN 消息发送后在指定时间内未收到响应时抛出此异常。
    可能的原因：
    - 电机未连接或断电
    - CAN 总线故障
    - 电机ID配置错误
    - 网络拥堵

    参数:
        motor_id (int): 超时的电机ID
        timeout (float): 超时时间（秒）
    """
    def __init__(self, motor_id=None, timeout=None, message=None):
        if message is None:
            msg_parts = ["CAN 通信超时"]
            if motor_id is not None:
                msg_parts.append(f"电机ID: {motor_id}")
            if timeout is not None:
                msg_parts.append(f"超时: {timeout}s")
            message = " - ".join(msg_parts)
        super().__init__(message)
        self.motor_id = motor_id
        self.timeout = timeout


class CANTransmissionError(CANError):
    """
    CAN 消息发送失败异常

    当 CAN 消息无法发送到总线时抛出此异常。
    可能的原因：
    - CAN 总线错误
    - 总线关闭状态
    - 硬件故障
    """
    pass


# ==================== 电机相关异常 ====================

class MotorError(OpenArmXError):
    """
    电机错误基类

    所有与电机控制和状态相关的异常都继承自此类。
    """
    pass


class MotorNotEnabledError(MotorError):
    """
    电机未使能异常

    当尝试控制未使能的电机时抛出此异常。

    参数:
        motor_id (int): 未使能的电机ID

    示例:
        >>> # 在未使能的情况下尝试移动电机
        >>> arm.move_joint_position(motor_id=5, position=1.0)
        >>> # 抛出 MotorNotEnabledError
    """
    def __init__(self, motor_id, message=None):
        if message is None:
            message = f"电机 {motor_id} 未使能，请先调用 enable() 方法"
        super().__init__(message)
        self.motor_id = motor_id


class MotorFaultError(MotorError):
    """
    电机故障异常

    当电机报告故障状态时抛出此异常。

    参数:
        motor_id (int): 故障的电机ID
        fault_code (int): 故障代码
        fault_description (str): 故障描述

    故障类型：
        - 欠压故障 (bit 0)
        - 驱动故障 (bit 1)
        - 过温 (bit 2)
        - 磁编码故障 (bit 3)
        - 堵转过载故障 (bit 4)
        - 未标定 (bit 5)
    """
    def __init__(self, motor_id, fault_code=None, fault_description=None):
        msg_parts = [f"电机 {motor_id} 故障"]
        if fault_code is not None:
            msg_parts.append(f"故障代码: 0x{fault_code:02X}")
        if fault_description:
            msg_parts.append(f"描述: {fault_description}")
        message = " - ".join(msg_parts)
        super().__init__(message)
        self.motor_id = motor_id
        self.fault_code = fault_code
        self.fault_description = fault_description


class MotorTimeoutError(MotorError):
    """
    电机响应超时异常

    当电机在指定时间内未响应命令时抛出此异常。

    参数:
        motor_id (int): 超时的电机ID
        command (str): 超时的命令类型
        timeout (float): 超时时间（秒）
    """
    def __init__(self, motor_id, command=None, timeout=None):
        msg_parts = [f"电机 {motor_id} 响应超时"]
        if command:
            msg_parts.append(f"命令: {command}")
        if timeout is not None:
            msg_parts.append(f"超时: {timeout}s")
        message = " - ".join(msg_parts)
        super().__init__(message)
        self.motor_id = motor_id
        self.command = command
        self.timeout = timeout


class MotorCalibrationError(MotorError):
    """
    电机未标定异常

    当电机未完成标定就尝试使用时抛出此异常。

    参数:
        motor_id (int): 未标定的电机ID
    """
    def __init__(self, motor_id, message=None):
        if message is None:
            message = f"电机 {motor_id} 未标定，请先进行标定"
        super().__init__(message)
        self.motor_id = motor_id


# ==================== 配置相关异常 ====================

class ConfigurationError(OpenArmXError):
    """
    配置错误基类

    所有与配置相关的异常都继承自此类。
    """
    pass


class InvalidMotorIDError(ConfigurationError):
    """
    无效的电机ID异常

    当使用的电机ID不在有效范围内时抛出此异常。

    参数:
        motor_id: 无效的电机ID
        valid_range (tuple): 有效的ID范围，如 (1, 8)
    """
    def __init__(self, motor_id, valid_range=(1, 8)):
        message = f"无效的电机ID: {motor_id}，有效范围: {valid_range[0]}-{valid_range[1]}"
        super().__init__(message)
        self.motor_id = motor_id
        self.valid_range = valid_range


class InvalidModeError(ConfigurationError):
    """
    无效的控制模式异常

    当指定的控制模式不支持时抛出此异常。

    参数:
        mode: 无效的控制模式
        valid_modes (list): 有效的控制模式列表
    """
    def __init__(self, mode, valid_modes=None):
        if valid_modes is None:
            valid_modes = ['mit', 'csp', 'pp', 'speed', 'current']
        message = f"无效的控制模式: {mode}，有效模式: {valid_modes}"
        super().__init__(message)
        self.mode = mode
        self.valid_modes = valid_modes


class InvalidParameterError(ConfigurationError):
    """
    无效的参数异常

    当参数值不在有效范围内或格式错误时抛出此异常。

    参数:
        param_name (str): 参数名称
        param_value: 参数值
        reason (str): 无效的原因
    """
    def __init__(self, param_name, param_value, reason=None):
        msg_parts = [f"无效的参数 {param_name}: {param_value}"]
        if reason:
            msg_parts.append(f"原因: {reason}")
        message = " - ".join(msg_parts)
        super().__init__(message)
        self.param_name = param_name
        self.param_value = param_value
        self.reason = reason


class ConfigFileError(ConfigurationError):
    """
    配置文件错误异常

    当配置文件加载或解析失败时抛出此异常。

    参数:
        config_file (str): 配置文件路径
        reason (str): 错误原因
    """
    def __init__(self, config_file, reason=None):
        msg_parts = [f"配置文件错误: {config_file}"]
        if reason:
            msg_parts.append(f"原因: {reason}")
        message = " - ".join(msg_parts)
        super().__init__(message)
        self.config_file = config_file
        self.reason = reason


# ==================== 限制相关异常 ====================

class LimitExceededError(OpenArmXError):
    """
    超出限制基类

    所有与参数限制相关的异常都继承自此类。
    """
    pass


class PositionLimitError(LimitExceededError):
    """
    位置超限异常

    当设置的位置超出电机允许范围时抛出此异常。

    参数:
        motor_id (int): 电机ID
        position (float): 设置的位置值
        min_limit (float): 最小限制
        max_limit (float): 最大限制
    """
    def __init__(self, motor_id, position, min_limit, max_limit):
        message = (
            f"电机 {motor_id} 位置超限: {position:.3f} rad "
            f"(允许范围: {min_limit:.3f} ~ {max_limit:.3f} rad)"
        )
        super().__init__(message)
        self.motor_id = motor_id
        self.position = position
        self.min_limit = min_limit
        self.max_limit = max_limit


class VelocityLimitError(LimitExceededError):
    """
    速度超限异常

    当设置的速度超出电机允许范围时抛出此异常。

    参数:
        motor_id (int): 电机ID
        velocity (float): 设置的速度值
        min_limit (float): 最小限制
        max_limit (float): 最大限制
    """
    def __init__(self, motor_id, velocity, min_limit, max_limit):
        message = (
            f"电机 {motor_id} 速度超限: {velocity:.3f} rad/s "
            f"(允许范围: {min_limit:.3f} ~ {max_limit:.3f} rad/s)"
        )
        super().__init__(message)
        self.motor_id = motor_id
        self.velocity = velocity
        self.min_limit = min_limit
        self.max_limit = max_limit


class TorqueLimitError(LimitExceededError):
    """
    扭矩超限异常

    当设置的扭矩超出电机允许范围时抛出此异常。

    参数:
        motor_id (int): 电机ID
        torque (float): 设置的扭矩值
        min_limit (float): 最小限制
        max_limit (float): 最大限制
    """
    def __init__(self, motor_id, torque, min_limit, max_limit):
        message = (
            f"电机 {motor_id} 扭矩超限: {torque:.3f} Nm "
            f"(允许范围: {min_limit:.3f} ~ {max_limit:.3f} Nm)"
        )
        super().__init__(message)
        self.motor_id = motor_id
        self.torque = torque
        self.min_limit = min_limit
        self.max_limit = max_limit


class KpLimitError(LimitExceededError):
    """
    KP增益超限异常

    当设置的KP增益超出允许范围时抛出此异常。

    参数:
        motor_id (int): 电机ID
        kp (float): 设置的KP值
        min_limit (float): 最小限制
        max_limit (float): 最大限制
    """
    def __init__(self, motor_id, kp, min_limit, max_limit):
        message = (
            f"电机 {motor_id} KP增益超限: {kp:.3f} "
            f"(允许范围: {min_limit:.3f} ~ {max_limit:.3f})"
        )
        super().__init__(message)
        self.motor_id = motor_id
        self.kp = kp
        self.min_limit = min_limit
        self.max_limit = max_limit


class KdLimitError(LimitExceededError):
    """
    KD增益超限异常

    当设置的KD增益超出允许范围时抛出此异常。

    参数:
        motor_id (int): 电机ID
        kd (float): 设置的KD值
        min_limit (float): 最小限制
        max_limit (float): 最大限制
    """
    def __init__(self, motor_id, kd, min_limit, max_limit):
        message = (
            f"电机 {motor_id} KD增益超限: {kd:.3f} "
            f"(允许范围: {min_limit:.3f} ~ {max_limit:.3f})"
        )
        super().__init__(message)
        self.motor_id = motor_id
        self.kd = kd
        self.min_limit = min_limit
        self.max_limit = max_limit


# ==================== 连接相关异常 ====================

class ConnectionError(OpenArmXError):
    """
    连接错误异常

    当与硬件的连接出现问题时抛出此异常。
    """
    pass


class ConnectionLostError(ConnectionError):
    """
    连接丢失异常

    当与电机或CAN总线的连接丢失时抛出此异常。

    参数:
        device (str): 丢失连接的设备描述
    """
    def __init__(self, device=None):
        message = "连接丢失"
        if device:
            message += f": {device}"
        super().__init__(message)
        self.device = device


# ==================== 异常导出 ====================

__all__ = [
    # 基础异常
    'OpenArmXError',

    # CAN 通信异常
    'CANError',
    'CANInitializationError',
    'CANTimeoutError',
    'CANTransmissionError',

    # 电机异常
    'MotorError',
    'MotorNotEnabledError',
    'MotorFaultError',
    'MotorTimeoutError',
    'MotorCalibrationError',

    # 配置异常
    'ConfigurationError',
    'InvalidMotorIDError',
    'InvalidModeError',
    'InvalidParameterError',
    'ConfigFileError',

    # 限制异常
    'LimitExceededError',
    'PositionLimitError',
    'VelocityLimitError',
    'TorqueLimitError',
    'KpLimitError',
    'KdLimitError',

    # 连接异常
    'ConnectionError',
    'ConnectionLostError',
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   test_motor_one_by_one.py
@Time    :   2025/12/16 15:33:02
@Author  :   Wei Lindong 
@Version :   1.0
@Desc    :   测试所有电机逐个运动
'''

import sys
import os

# 添加 src 目录到 Python 路径
# tests/ 和 src/ 是同级目录，需要先回到父目录再进入 src
current_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(current_dir)
src_dir = os.path.join(parent_dir, 'src')
sys.path.insert(0, src_dir)

# 导入 Arm 类和异常
from openarmx_driver import Arm
import time

if __name__ == "__main__":
    # 运动参数（可调）
    kp = 10.0
    kd = 1.0
    red_max = 0.2

    # 初始化左右臂
    right_arm = Arm(can_channel='can0', side='right')
    left_arm = Arm(can_channel='can1', side='left')

    # 开启左右臂电机
    right_arm.enable_all()
    left_arm.enable_all()

    # 设置为 MIT 模式
    right_arm.set_mode('mit')
    left_arm.set_mode('mit')

    # 逐个运动电机
    for arm in [right_arm, left_arm]:
        for motor_id in arm.motor_ids:
            try:
                arm.move_joint_mit(motor_id, position=-red_max, kp=kp, kd=kd)
                time.sleep(0.5)
                arm.home_joint(motor_id, kp=kp, kd=kd)
                time.sleep(0.5)
            except KeyboardInterrupt:
                print("按下 Ctrl+C,结束----")
                right_arm.disable_all()
                left_arm.disable_all()
                break
    
    
    right_arm.close()
    left_arm.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   test_motor_all_random.py
@Time    :   2025/12/16
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   电机随机运动测试
'''

import sys
import os
import time
import random

# 添加 src 目录到 Python 路径
current_dir = os.path.dirname(os.path.abspath(__file__))
parent_dir = os.path.dirname(current_dir)
src_dir = os.path.join(parent_dir, 'src')
sys.path.insert(0, src_dir)

from openarmx_driver import Robot


if __name__ == "__main__":
    # 参数配置
    red_max = 0.3
    time_delay = 0.005
    kp_list = [50, 50, 50, 50, 5, 5, 5, 5]
    kd_list = [5, 5, 5, 5, 0.25, 0.25, 0.25, 0.25]

    # 初始化机器人
    robot = Robot(left_canid='can0', right_canid='can1')

    try:
        # 使能所有电机
        robot.enable_all()
        time.sleep(0.1)

        # 设置运控模式
        robot.set_mode_all('mit')
        time.sleep(0.1)

        # 随机运动循环
        test_id = 0
        while True:
            print(f'test_id:{test_id}')
            test_id += 1

            # 生成随机位置列表
            positions = [
                random.uniform(0.2 * red_max, red_max)
                for _ in range(7)
            ]

            # 左右臂同时移动到随机位置
            for i, pos in enumerate(positions):
                motor_id = i + 1
                kp = kp_list[motor_id - 1] / 2
                kd = kd_list[motor_id - 1] / 2
                robot.left_arm.move_joint_mit(motor_id, position=pos, kp=kp, kd=kd)
                robot.right_arm.move_joint_mit(motor_id, position=pos, kp=kp, kd=kd)
                time.sleep(time_delay)
            time.sleep(1)

            # 回到零位
            robot.move_all_to_zero_mit(kp=5.0, kd=0.5)
            time.sleep(0.5)

    except KeyboardInterrupt:
        print("\n用户中断")
    except Exception as e:
        print(f"错误: {e}")
    finally:
        robot.shutdown()



from setuptools import setup
from setuptools.dist import Distribution


class BinaryDistribution(Distribution):
    """Distribution which always forces a binary package with platform name"""
    def has_ext_modules(self):
        return True


try:
    from wheel.bdist_wheel import bdist_wheel as _bdist_wheel

    class bdist_wheel(_bdist_wheel):
        def finalize_options(self):
            _bdist_wheel.finalize_options(self)
            # Force manylinux platform tag
            self.plat_name_supplied = True
            self.plat_name = 'manylinux_2_17_x86_64'

    cmdclass = {'bdist_wheel': bdist_wheel}
except ImportError:
    cmdclass = {}


setup(
    distclass=BinaryDistribution,
    cmdclass=cmdclass,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   arm.py
@Time    :   2025/12/16
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   OpenArmX 机械臂控制类 - 主要对外接口
'''

import can
import time
from typing import Optional, List, Dict, Union, Tuple

# 导入异常类
from .exceptions import (
    CANInitializationError,
    InvalidMotorIDError,
    InvalidModeError,
)

# 导入内部模块
from ._lib.motor_manager import (
    enable_motor,
    disable_motor,
    enable_all_motors,
    disable_all_motors,
    set_control_mode,
    get_motor_status_readonly,
    get_motor_basic_telemetry,
    set_motor_zero,
    set_zero_sta_parameter,
    read_motor_parameter
)

from ._lib.motor_control import (
    mit_motion_control,
    mit_motion_control_simple,
    mit_zero_position,
    mit_velocity_control,
    mit_torque_control,
    csp_motion_control,
    csp_set_speed_limits,
    csp_move_to_flowwork
)

from ._lib.motor_config_loader import MotorConfigLoader
from ._lib.log_utils import log_output

class Arm:
    """
    OpenArmX 机械臂控制类

    这是 openarmx_driver 包的主要对外接口，用于控制基于 Robstride 电机的机械臂。
    每条机械臂有8个电机，通过 CAN 总线进行通信。

    电机配置映射:
        - 电机 1-2: RS04 型号 (大扭矩关节)
        - 电机 3-4: RS03 型号 (中扭矩关节)
        - 电机 5-8: RS00 型号 (小扭矩关节/夹爪)

    参数:
        can_channel (str): CAN 通道名称，如 'can0', 'can1' 等
        side (str, optional): 机械臂方向
            - 'left': 左臂（默认所有电机方向反转）
            - 'right': 右臂（默认所有电机方向正常）
            - None: 不使用方向配置（所有电机方向正常）
        bustype (str): CAN 总线类型，默认 'socketcan'
        bitrate (int): CAN 总线波特率，默认 1000000 (1Mbps)
        motor_ids (List[int]): 要控制的电机ID列表，默认 [1,2,3,4,5,6,7,8]
        direction_multipliers (Dict[int, float], optional): 自定义每个电机的方向系数
            - 1.0: 正常方向
            - -1.0: 反转方向
            如果提供此参数，将覆盖 side 参数的默认配置

    示例:
        >>> # 创建右臂实例
        >>> right_arm = Arm('can0', side='right')
        >>>
        >>> # 创建左臂实例（电机方向自动反转）
        >>> left_arm = Arm('can1', side='left')
        >>>
        >>> # 自定义方向配置（只反转电机1和2）
        >>> custom_arm = Arm('can0', direction_multipliers={1: -1, 2: -1, 3: 1, 4: 1, 5: 1, 6: 1, 7: 1, 8: 1})
        >>>
        >>> # 使能所有电机
        >>> right_arm.enable_all()
        >>>
        >>> # 设置为 MIT 模式并移动电机
        >>> right_arm.set_mode('mit')
        >>> right_arm.move_joint_mit(motor_id=5, position=1.0, kp=20, kd=2)
        >>>
        >>> # 获取电机状态
        >>> status = right_arm.get_status(motor_id=5)
        >>> print(f"角度: {status['angle']}, 速度: {status['velocity']}")
        >>>
        >>> # 停止所有电机
        >>> right_arm.disable_all()
        >>>
        >>> # 关闭连接
        >>> right_arm.close()
    """

    def __init__(self,
                 can_channel: str,
                 side: Optional[str] = None,
                 bustype: str = 'socketcan',
                 bitrate: int = 1000000,
                 motor_ids: Optional[List[int]] = None,
                 direction_multipliers: Optional[Dict[int, float]] = None,
                 log=None):
        """
        初始化机械臂控制器

        参数:
            can_channel (str): CAN 通道名称，如 'can0', 'can1'
            side (str, optional): 机械臂方向 ('left', 'right', None)
            bustype (str): CAN 总线类型，默认 'socketcan'
            bitrate (int): CAN 总线波特率，默认 1000000
            motor_ids (List[int]): 要控制的电机ID列表，默认 [1,2,3,4,5,6,7,8]
            direction_multipliers (Dict[int, float], optional): 自定义方向系数
        """
        self.can_channel = can_channel
        self.side = side
        self.bustype = bustype
        self.bitrate = bitrate
        self.motor_ids = motor_ids if motor_ids is not None else list(range(1, 9))
        self.log = log

        # 初始化 CAN 总线
        try:
            self.bus = can.Bus(
                channel=can_channel,
                bustype=bustype,
                bitrate=bitrate
            )
            side_info = f" ({side}臂)" if side else ""
            log_output(f"[Arm] CAN 总线 {can_channel}{side_info} 初始化成功", "SUCCESS", self.log)
        except Exception as e:
            raise CANInitializationError(
                f"CAN 总线 {can_channel} 初始化失败: {e}\n"
                f"请检查:\n"
                f"  1. CAN 接口是否存在 (ip link show {can_channel})\n"
                f"  2. CAN 接口是否已启动 (sudo ip link set {can_channel} up type can bitrate {bitrate})\n"
                f"  3. 是否有足够的权限\n"
                f"  4. CAN 驱动是否已加载 (lsmod | grep can)"
            )

        # 加载电机配置
        self.config_loader = MotorConfigLoader()

        # 设置方向系数
        self._setup_direction_multipliers(direction_multipliers)

        # 当前控制模式记录 (用于用户参考)
        self._current_mode: Dict[int, str] = {}

        log_output(f"[Arm] 机械臂初始化完成，电机ID: {self.motor_ids}", level="SUCCESS", log=log)

    def _setup_direction_multipliers(self, custom_multipliers: Optional[Dict[int, float]] = None):
        """
        设置电机方向系数

        参数:
            custom_multipliers (Dict[int, float], optional): 自定义方向系数
        """
        # 如果提供了自定义方向系数，直接使用
        if custom_multipliers is not None:
            self.direction_multipliers = custom_multipliers
            log_output(f"[Arm] 使用自定义方向配置: {custom_multipliers}", "INFO", self.log)
            return

        # 根据 side 参数从配置文件加载方向系数
        if self.side in ['left', 'right']:
            # 从配置加载器获取方向系数
            self.direction_multipliers = self.config_loader.get_direction_multipliers(
                self.side, self.motor_ids
            )
            log_output(f"[Arm] {self.side}臂模式：从配置文件加载方向系数", "INFO", self.log)
            if len(self.direction_multipliers) > 0:
                # 显示方向配置摘要
                reversed_motors = [mid for mid, mult in self.direction_multipliers.items() if mult < 0]
                if reversed_motors:
                    log_output(f"[Arm] 反转电机: {reversed_motors}", "INFO", self.log)
                else:
                    log_output(f"[Arm] 所有电机方向正常", "INFO", self.log)
        else:
            # 未指定方向：所有电机方向正常
            self.direction_multipliers = {i: 1.0 for i in self.motor_ids}

    def _validate_motor_id(self, motor_id: int):
        """
        验证电机ID是否有效

        参数:
            motor_id (int): 电机ID

        抛出:
            InvalidMotorIDError: 如果电机ID无效
        """
        if motor_id not in self.motor_ids:
            raise InvalidMotorIDError(
                motor_id,
                valid_range=(min(self.motor_ids), max(self.motor_ids))
            )

    def _apply_direction(self, motor_id: int, value: float) -> float:
        """
        应用方向系数到数值

        参数:
            motor_id (int): 电机ID
            value (float): 原始数值（位置、速度或扭矩）

        返回:
            float: 应用方向系数后的数值
        """
        multiplier = self.direction_multipliers.get(motor_id, 1.0)
        return value * multiplier

    def _reverse_direction(self, motor_id: int, value: float) -> float:
        """
        反向应用方向系数（用于读取反馈值）

        参数:
            motor_id (int): 电机ID
            value (float): 反馈数值

        返回:
            float: 反向应用方向系数后的数值
        """
        multiplier = self.direction_multipliers.get(motor_id, 1.0)
        return value * multiplier  # 乘法是对称的，所以反向也是乘以相同系数

    # ==================== 基础控制方法 ====================

    def enable(self, motor_id: int, timeout: float = 1.0, verbose: bool = False) -> int:
        """
        使能指定电机

        参数:
            motor_id (int): 电机ID (1-8)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败

        抛出:
            InvalidMotorIDError: 如果电机ID无效
        """
        self._validate_motor_id(motor_id)
        return enable_motor(self.bus, motor_id, timeout=timeout, verbose=verbose)

    def disable(self, motor_id: int, timeout: float = 1.0, verbose: bool = False) -> int:
        """
        停止指定电机

        参数:
            motor_id (int): 电机ID (1-8)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败

        抛出:
            InvalidMotorIDError: 如果电机ID无效
        """
        self._validate_motor_id(motor_id)
        return disable_motor(self.bus, motor_id, timeout=timeout, verbose=verbose)

    def enable_all(self, verbose: bool = False) -> Dict[int, int]:
        """
        使能所有电机

        参数:
            verbose (bool): 是否打印详细信息

        返回:
            Dict[int, int]: {motor_id: state} 每个电机的状态
        """
        return enable_all_motors(self.bus, motor_ids=self.motor_ids, verbose=verbose)

    def disable_all(self, verbose: bool = False) -> Dict[int, int]:
        """
        停止所有电机

        参数:
            verbose (bool): 是否打印详细信息

        返回:
            Dict[int, int]: {motor_id: state} 每个电机的状态
        """
        return disable_all_motors(self.bus, motor_ids=self.motor_ids, verbose=verbose)

    # ==================== 模式设置 ====================

    def set_mode(self, mode: Union[str, int],
                 motor_id: Optional[int] = None,
                 timeout: float = 1.0,
                 verbose: bool = False) -> int:
        """
        设置电机控制模式

        参数:
            mode (str|int): 控制模式
                - 'mit' / 0: MIT 运控模式
                - 'csp' / 5: CSP 位置模式
                - 'pp' / 1: PP 位置模式
                - 'speed' / 2: 速度模式
                - 'current' / 3: 电流模式
            motor_id (int, optional): 电机ID，如果为 None 则设置所有电机
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 非0=失败（单个电机）或失败次数（所有电机）

        抛出:
            InvalidMotorIDError: 如果电机ID无效
            InvalidModeError: 如果控制模式无效
        """
        # 验证模式
        valid_modes = ['mit', 'pp', 'speed', 'current', 'csp', 0, 1, 2, 3, 5]
        if mode not in valid_modes:
            raise InvalidModeError(mode, valid_modes=['mit', 'pp', 'speed', 'current', 'csp'])

        if motor_id is not None:
            # 验证电机ID
            self._validate_motor_id(motor_id)

            # 设置单个电机
            state = set_control_mode(self.bus, motor_id, mode=mode,
                                    timeout=timeout, verbose=verbose)
            if state == 0:
                self._current_mode[motor_id] = str(mode)
            return state
        else:
            # 设置所有电机
            failed_count = 0
            for mid in self.motor_ids:
                state = set_control_mode(self.bus, mid, mode=mode,
                                        timeout=timeout, verbose=verbose)
                if state == 0:
                    self._current_mode[mid] = str(mode)
                else:
                    failed_count += 1
                time.sleep(0.01)
            return failed_count

    # ==================== 状态查询 ====================

    def get_status(self, motor_id: int,
                   timeout: float = 1.0,
                   verbose: bool = False) -> Optional[Dict]:
        """
        获取电机状态

        参数:
            motor_id (int): 电机ID (1-8)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            Dict 或 None: 包含 angle, velocity, torque, temperature,
                         mode_status, fault_status 等字段
        """
        state, info = get_motor_status_readonly(
            self.bus, motor_id,
            timeout=timeout, verbose=verbose
        )

        # 应用反向方向系数到反馈值
        if state == 0 and info is not None:
            if 'angle' in info:
                info['angle'] = self._reverse_direction(motor_id, info['angle'])
            if 'velocity' in info:
                info['velocity'] = self._reverse_direction(motor_id, info['velocity'])
            if 'torque' in info:
                info['torque'] = self._reverse_direction(motor_id, info['torque'])

        return info if state == 0 else None

    def get_telemetry(self, motor_id: int,
                     timeout: float = 1.0,
                     verbose: bool = False) -> Optional[Dict]:
        """
        获取电机基本遥测数据

        参数:
            motor_id (int): 电机ID (1-8)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            Dict 或 None: 包含 mech_pos, mech_vel, iqf, vbus 等字段
        """
        state, info = get_motor_basic_telemetry(
            self.bus, motor_id,
            timeout=timeout, verbose=verbose
        )

        # 应用反向方向系数到反馈值
        if state == 0 and info is not None:
            if 'mech_pos' in info:
                info['mech_pos'] = self._reverse_direction(motor_id, info['mech_pos'])
            if 'mech_vel' in info:
                info['mech_vel'] = self._reverse_direction(motor_id, info['mech_vel'])
            # iqf 和 vbus 不需要反转

        return info if state == 0 else None

    def get_all_status(self, timeout: float = 1.0, verbose: bool = False) -> Dict[int, Optional[Dict]]:
        """
        获取所有电机的状态

        参数:
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            Dict[int, Dict]: {motor_id: status_dict}
        """
        results = {}
        for motor_id in self.motor_ids:
            results[motor_id] = self.get_status(motor_id, timeout=timeout, verbose=verbose)
        return results

    def show_motor_status(self, motor_id: Optional[int] = None,
                         show_header: bool = True) -> None:
        """
        显示电机状态（格式化输出）

        参数:
            motor_id (int, optional): 电机ID，如果为None则显示所有电机
            show_header (bool): 是否显示表头

        示例:
            >>> arm.show_motor_status(3)  # 显示单个电机
            >>> arm.show_motor_status()    # 显示所有电机
        """
        # 打印表头
        if show_header:
            log_output("="*120, "INFO", self.log)
            log_output("电机状态", "INFO", self.log)
            log_output("="*120, "INFO", self.log)
            log_output("ID | 角度(rad) | 速度(rad/s) | 力矩(Nm) |  温度    | 模式              | 状态", "INFO", self.log)
            log_output("-"*120, "INFO", self.log)

        # 获取状态信息
        if motor_id is not None:
            # 显示单个电机
            try:
                info = self.get_status(motor_id)
                if info:
                    self._print_single_motor_status(motor_id, info)
                else:
                    log_output(f"ID:{motor_id:2d} | ✗ 无响应 - 电机未连接或已断电", "ERROR", self.log)
            except Exception as e:
                log_output(f"ID:{motor_id:2d} | ⚠ 异常 - {str(e)}", "ERROR", self.log)
        else:
            # 显示所有电机
            for mid in self.motor_ids:
                try:
                    info = self.get_status(mid)
                    if info:
                        self._print_single_motor_status(mid, info)
                    else:
                        log_output(f"ID:{mid:2d} | ✗ 无响应", "WARNING", self.log)
                except Exception as e:
                    log_output(f"ID:{mid:2d} | ⚠ 异常 - {str(e)}", "WARNING", self.log)

        if show_header:
            log_output("="*120, "INFO", self.log)

    def _print_single_motor_status(self, motor_id: int, info: dict) -> None:
        """
        打印单个电机的状态信息（内部辅助函数）

        参数:
            motor_id (int): 电机ID
            info (dict): 状态信息字典
        """
        # 根据模式状态选择图标
        mode_status = info.get('mode_status', '未知')
        if 'Motor模式' in mode_status or '运行' in mode_status:
            mode_icon = "🟢"
        elif 'Reset模式' in mode_status or '复位' in mode_status:
            mode_icon = "🔴"
        elif 'Cali模式' in mode_status or '标定' in mode_status:
            mode_icon = "🟡"
        else:
            mode_icon = "⚪"

        # 根据故障状态选择图标
        fault_status = info.get('fault_status', '未知')
        if fault_status == "正常":
            fault_icon = "✓"
        else:
            fault_icon = "⚠"

        # 格式化输出
        log_output(f"ID:{motor_id:2d} | "
                   f"{info.get('angle', 0.0):9.3f} | "
                   f"{info.get('velocity', 0.0):11.3f} | "
                   f"{info.get('torque', 0.0):8.3f} | "
                   f"{info.get('temperature', 0.0):5.1f}°C | "
                   f"{mode_icon} {mode_status:15s} | "
                   f"{fault_icon} {fault_status}",
                   "INFO", self.log)

    # ==================== MIT 模式控制 ====================

    def move_joint_mit(self, motor_id: int,
                      position: float = 0.0,
                      velocity: float = 0.0,
                      torque: float = 0.0,
                      kp: float = 0.0,
                      kd: float = 0.0,
                      wait_response: bool = False,
                      timeout: float = 1.0,
                      verbose: bool = False) -> int:
        """
        MIT 模式运动控制

        参数:
            motor_id (int): 电机ID (1-8)
            position (float): 目标位置 (弧度)
            velocity (float): 目标速度 (弧度/秒)
            torque (float): 前馈扭矩 (牛米)
            kp (float): 位置增益
            kd (float): 速度增益
            wait_response (bool): 是否等待响应
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败

        抛出:
            InvalidMotorIDError: 如果电机ID无效
        """
        # 验证电机ID
        self._validate_motor_id(motor_id)

        # 应用方向系数
        position_cmd = self._apply_direction(motor_id, position)
        velocity_cmd = self._apply_direction(motor_id, velocity)
        torque_cmd = self._apply_direction(motor_id, torque)

        state, _, _ = mit_motion_control(
            self.bus, motor_id,
            position=position_cmd, velocity=velocity_cmd, torque=torque_cmd,
            kp=kp, kd=kd,
            wait_response=wait_response,
            timeout=timeout, verbose=verbose
        )
        return state

    def move_joint_position(self, motor_id: int,
                           position: float,
                           kp: float = 5.0,
                           kd: float = 0.5,
                           verbose: bool = False) -> int:
        """
        MIT 模式简化位置控制

        参数:
            motor_id (int): 电机ID (1-8)
            position (float): 目标位置 (弧度)
            kp (float): 位置增益
            kd (float): 速度增益
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        # 应用方向系数
        position_cmd = self._apply_direction(motor_id, position)

        return mit_motion_control_simple(
            self.bus, motor_id, position_cmd,
            kp=kp, kd=kd, verbose=verbose
        )

    def move_joint_velocity(self, motor_id: int,
                           velocity: float,
                           kd: float = 2.0,
                           verbose: bool = False) -> int:
        """
        MIT 模式速度控制

        参数:
            motor_id (int): 电机ID (1-8)
            velocity (float): 目标速度 (弧度/秒)
            kd (float): 速度增益
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        # 应用方向系数
        velocity_cmd = self._apply_direction(motor_id, velocity)

        return mit_velocity_control(
            self.bus, motor_id, velocity_cmd,
            kd=kd, verbose=verbose
        )

    def move_joint_torque(self, motor_id: int,
                         torque: float,
                         verbose: bool = False) -> int:
        """
        MIT 模式扭矩控制

        参数:
            motor_id (int): 电机ID (1-8)
            torque (float): 目标扭矩 (牛米)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        # 应用方向系数
        torque_cmd = self._apply_direction(motor_id, torque)

        return mit_torque_control(
            self.bus, motor_id, torque_cmd,
            verbose=verbose
        )

    def home_joint(self, motor_id: int,
                  kp: float = 5.0,
                  kd: float = 0.5,
                  verbose: bool = False) -> int:
        """
        将电机移动到零位

        参数:
            motor_id (int): 电机ID (1-8)
            kp (float): 位置增益
            kd (float): 速度增益
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        return mit_zero_position(
            self.bus, motor_id,
            kp=kp, kd=kd, verbose=verbose
        )

    def home_all(self, kp: float = 5.0, kd: float = 0.5, verbose: bool = False) -> Dict[int, int]:
        """
        所有电机归零

        参数:
            kp (float): 位置增益
            kd (float): 速度增益
            verbose (bool): 是否打印详细信息

        返回:
            Dict[int, int]: {motor_id: state}
        """
        results = {}
        for motor_id in self.motor_ids:
            results[motor_id] = self.home_joint(motor_id, kp=kp, kd=kd, verbose=verbose)
            time.sleep(0.01)
        return results

    # ==================== CSP 模式控制 ====================

    def move_joint_csp(self, motor_id: int,
                      position: float,
                      wait_response: bool = False,
                      timeout: float = 0.2,
                      verbose: bool = False) -> int:
        """
        CSP 模式位置控制

        参数:
            motor_id (int): 电机ID (1-8)
            position (float): 目标位置 (弧度)
            wait_response (bool): 是否等待响应
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        # 应用方向系数
        position_cmd = self._apply_direction(motor_id, position)

        return csp_motion_control(
            self.bus, motor_id, position_cmd,
            wait_response=wait_response,
            timeout=timeout, verbose=verbose
        )

    def set_csp_limits(self, motor_id: int,
                      speed_limit: Optional[float] = None,
                      current_limit: Optional[float] = None,
                      timeout: float = 0.2,
                      verbose: bool = False) -> int:
        """
        设置 CSP 模式的速度/电流限制

        参数:
            motor_id (int): 电机ID (1-8)
            speed_limit (float, optional): 速度限制 (弧度/秒)
            current_limit (float, optional): 电流限制 (牛米)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 非0=失败
        """
        return csp_set_speed_limits(
            self.bus, motor_id,
            speed_limit=speed_limit,
            current_limit=current_limit,
            timeout=timeout, verbose=verbose
        )

    def move_to_csp(self, motor_id: int,
                   position: float,
                   speed_limit: Optional[float] = None,
                   current_limit: Optional[float] = None,
                   timeout: float = 1.0,
                   verbose: bool = False) -> int:
        """
        CSP 完整工作流程：设置模式、限制并移动到目标位置

        参数:
            motor_id (int): 电机ID (1-8)
            position (float): 目标位置 (弧度)
            speed_limit (float, optional): 速度限制 (弧度/秒)
            current_limit (float, optional): 电流限制 (牛米)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        # 应用方向系数
        position_cmd = self._apply_direction(motor_id, position)

        return csp_move_to_flowwork(
            self.bus, motor_id, position_cmd,
            speed_limit=speed_limit,
            current_limit=current_limit,
            timeout=timeout, verbose=verbose
        )

    # ==================== 零点设置 ====================

    def set_zero(self, motor_id: int,
                timeout: float = 1.0,
                verbose: bool = False) -> int:
        """
        设置电机当前位置为零点

        参数:
            motor_id (int): 电机ID (1-8)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        return set_motor_zero(self.bus, motor_id, timeout=timeout, verbose=verbose)

    def set_zero_range(self, motor_id: int,
                      zero_sta: int,
                      timeout: float = 1.0,
                      verbose: bool = False) -> int:
        """
        设置零点表示范围

        参数:
            motor_id (int): 电机ID (1-8)
            zero_sta (int): 0=范围 0~2π, 1=范围 -π~π
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            int: 0=成功, 1=失败
        """
        return set_zero_sta_parameter(
            self.bus, motor_id, zero_sta,
            timeout=timeout, verbose=verbose
        )

    # ==================== 参数读取 ====================

    def read_parameter(self, motor_id: int,
                      param_index: int,
                      timeout: float = 1.0,
                      verbose: bool = False) -> Tuple[int, any]:
        """
        读取电机参数

        参数:
            motor_id (int): 电机ID (1-8)
            param_index (int): 参数索引 (如 0x7019)
            timeout (float): 超时时间(秒)
            verbose (bool): 是否打印详细信息

        返回:
            Tuple[int, any]: (state, value) state=0表示成功
        """
        return read_motor_parameter(
            self.bus, motor_id, param_index,
            timeout=timeout, verbose=verbose
        )

    # ==================== 配置查询 ====================

    def get_motor_config(self, motor_id: int) -> Optional[Dict]:
        """
        获取电机配置参数

        参数:
            motor_id (int): 电机ID (1-8)

        返回:
            Dict: 电机配置字典
        """
        return self.config_loader.get_motor_config(motor_id)

    def get_motor_limits(self, motor_id: int) -> Dict:
        """
        获取电机所有限制参数

        参数:
            motor_id (int): 电机ID (1-8)

        返回:
            Dict: 包含 P_MIN, P_MAX, T_MIN, T_MAX 等限制参数
        """
        return self.config_loader.get_all_limits(motor_id)

    # ==================== 资源管理 ====================

    def close(self):
        """关闭 CAN 总线连接"""
        if hasattr(self, 'bus') and self.bus is not None:
            self.bus.shutdown()
            log_output(f"[Arm] CAN 总线 {self.can_channel} 已关闭", "SUCCESS", self.log)

    def __enter__(self):
        """支持 with 语句"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """支持 with 语句自动关闭"""
        self.close()

    def __repr__(self):
        side_str = f", side='{self.side}'" if self.side else ""
        return f"Arm(can_channel='{self.can_channel}'{side_str}, motor_ids={self.motor_ids})"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
@File    :   __init__.py
@Time    :   2025/12/16
@Author  :   Wei Lindong
@Version :   1.0
@Desc    :   Configuration files for OpenArmX driver
'''

# This package contains configuration files
# Configuration is loaded automatically by the Arm class
